<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Oct 21, 2020
 */

/**
 * Siteworx Daily Backups plugin.
 *
 * @package    InterWorx
 * @subpackage Plugin
 */
class Plugin_SiteworxBackupsCron extends Plugin
{
    const DEFAULT_BACKUPS_LOCATION = '/backup';
    const DEFAULT_KEEP_DAILY = 4;
    const DEFAULT_KEEP_WEEKLY = 3;
    const DEFAULT_KEEP_MONTHLY = 2;
    const DEFAULT_AVAIL = 5;

    const INPUT_DIRECTORY = 'directory';
    const INPUT_KEEP_DAILY = 'daily';
    const INPUT_KEEP_WEEKLY = 'weekly';
    const INPUT_KEEP_MONTHLY = 'monthly';
    const INPUT_SPACE = 'space';

    private $_scriptPath;
    private $_crontab;
    private $_backupsCronJobs;

    /**
     * Decides which category to display plugin under.
     *
     * @return string
     */
    public function getCategory()
    {
        return Plugin_Category::BACKUP_RESTORE;
    }

    /**
     * Specify inputs etc to show when the plugin is edited.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    public function initializeMyEditForm(Form_NW_Plugins_Edit $Form)
    {
        $pluginsPath = Ini::get(Ini::IWORX_PLUGINS, 'dir');
        $this->_scriptPath = "$pluginsPath/siteworx-backups-cron/scripts/iwx-backup.sh";
        $this->_crontab = IWorxCron::getUserCrontab('root', true);
        $this->_backupsCronJobs = $this->_getBackupsCronJobs();

        $this->_configureForm($Form);
    }

    /**
     * Configure form to include input for backups path &
     * keep daily/weekly/monthly
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    private function _configureForm(Form_NW_Plugins_Edit $Form)
    {
        $defaultValues = $this->_getDefaultValues();
        $defaultDir = $defaultValues[self::INPUT_DIRECTORY];
        $defaultKeepDaily = $defaultValues[self::INPUT_KEEP_DAILY];
        $defaultKeepWeekly = $defaultValues[self::INPUT_KEEP_WEEKLY];
        $defaultKeepMonthly = $defaultValues[self::INPUT_KEEP_MONTHLY];
        $defaultSpace = $defaultValues[self::INPUT_SPACE];

        $Group = $Form->addGroup('config', '##LG_CONFIGURATION##');

        $DirInput = new Input_String('directory', $defaultDir);
        $DirInput->addValidator(new IWorx_Validate_Directory_Path());

        $Group->addInput($DirInput)
        ->setLabel('Backups Path');

        $Group->addInput(new Input_Number(self::INPUT_KEEP_DAILY, $defaultKeepDaily))
        ->setLabel('Keep Daily');

        $Group->addInput(new Input_Number(self::INPUT_KEEP_WEEKLY, $defaultKeepWeekly))
        ->setLabel('Keep Weekly');

        $Group->addInput(new Input_Number(self::INPUT_KEEP_MONTHLY, $defaultKeepMonthly))
        ->setLabel('Keep Monthly');

        $Group->addInput(new Input_Number(self::INPUT_SPACE, $defaultSpace))
        ->setLabel('Minimum Available Disk Space');
    }

    /**
     * Get default values by parsing script command.
     *
     * @return array
     */
    private function _getDefaultValues()
    {
        $keepDaily = self::DEFAULT_KEEP_DAILY;
        $keepWeekly = self::DEFAULT_KEEP_WEEKLY;
        $keepMonthly = self::DEFAULT_KEEP_MONTHLY;
        $path = self::DEFAULT_BACKUPS_LOCATION;
        $space = self::DEFAULT_AVAIL;

        if ($this->_cronJobExists()) {
            $defaults = explode(' ', $this->_backupsCronJobs[0]['script']);
            $keepDaily = $defaults[2];
            $keepWeekly = $defaults[3];
            $keepMonthly = $defaults[4];
            $path = $defaults[5];
            $space = $defaults[6];
        }

        return array(
        self::INPUT_DIRECTORY => $path,
        self::INPUT_SPACE => $space,
        self::INPUT_KEEP_DAILY => $keepDaily,
        self::INPUT_KEEP_WEEKLY => $keepWeekly,
        self::INPUT_KEEP_MONTHLY => $keepMonthly
        );
    }

    /**
     * Get cron jobs executing backup script.
     *
     * @return array
     */
    private function _getBackupsCronJobs()
    {
        if (isset($this->_crontab) && !empty($this->_crontab['jobs'])) {
            return $this->_getDuplicateJobs($this->_crontab['jobs']);
        }
    }

    /**
     * @param array $cronJobs
     * @return array
     */
    private function _getDuplicateJobs(array $cronJobs)
    {
        $duplicates = array();

        foreach ($cronJobs as $cronJob) {
            if (strpos($cronJob['script'], $this->_scriptPath) !== false) {
                $duplicates[] = $cronJob;
            }
        }

        return $duplicates;
    }

    /**
     * Add task to daily cron if plugin is being enabled.
     *
     * @param Form $Form
     */
    public function validateMyEditForm(Form $Form)
    {
        try {
            $this->_processRequest($Form);
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    /**
     * Process request as part of validate
     * in order to be able to catch any errors.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    private function _processRequest(Form_NW_Plugins_Edit $Form)
    {
        if ($Form->enableRequest()) {
            $this->_setUpCronJob($Form);
        }

        if (!$Form->enableRequest() && $this->_cronJobExists()) {
            $this->_deleteBackupsCronJobs();
        }
    }

    /**
     * Set up cron job.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    private function _setUpCronJob(Form_NW_Plugins_Edit $Form)
    {
        if ($this->_cronJobExists()) {
            $this->_deleteBackupsCronJobs();
        }

        $this->_addCronJob($Form);
    }

    /**
     * @return boolean
     */
    private function _cronJobExists()
    {
        return !empty($this->_backupsCronJobs);
    }

    /**
     * add cron job.
     *
     * @param Form_NW_Plugins_Edit $Form
     */
    private function _addCronJob(Form_NW_Plugins_Edit $Form)
    {
        $backupsPath = $Form->getValue(self::INPUT_DIRECTORY);
        $usageSpace = $Form->getValue(self::INPUT_SPACE);
        $keepDaily = $Form->getValue(self::INPUT_KEEP_DAILY);
        $keepWeekly = $Form->getValue(self::INPUT_KEEP_WEEKLY);
        $keepMonthly = $Form->getValue(self::INPUT_KEEP_MONTHLY);

        $args = array(
        'user' => 'root',
        'hour' => array('2'),
        'minute' => array('0'),
        'day' => '*', // every day
        'month' => '*', // every month
        'script' => "sh $this->_scriptPath $keepDaily $keepWeekly $keepMonthly $backupsPath $usageSpace",
        );

        $Route = IW::FC()->routeFromPHP(
            'Ctrl_Nodeworx_Cron',
            'addCommit',
            $args,
            IW::NW()
        );

        $errors = $Route->getErrors();

        if (!empty($errors)) {
            $message = 'Failed to set up cron job';
            IW::getLog()->error($message, $errors);

            throw new IWorx_Exception_Plugin('Failed to set up cron job');
        }
    }


    /**
     * Delete cron jobs running backups script.
     */
    private function _deleteBackupsCronJobs()
    {
        foreach ($this->_backupsCronJobs as $cronJob) {
            $linenum = $cronJob['linenum'];
            unset($this->_crontab['jobs'][$linenum]);
        }

        $retval = IWorxCron::installCrontab($this->_crontab, 'root', $result);

        if ($retval != CLI::OK) {
            $message = 'Failed deleting cron job';
            IW::getLog()->error($message);
            throw new IWorx_Exception_Plugin($message);
        }
    }
}
